/*
	RC4 Documents - Hash functions for old Word, Excel, PDF documents.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include "common.h"
#include "excel97.h"
#include "pdf40.h"
#include "word97.h"

typedef void (*RC4_HASH_FUNC)(const void *key, void *hash);

__inline__ void reduxKey(void *key, uint32_t chainPos, uint32_t tableIndex)
{
	((uint32_t*) key)[0] += chainPos + (tableIndex << 24);
}

void generateChain(RC4_HASH_FUNC rc4HashFunc, const void *startPoint, void *endpoint, uint32_t chainLength, uint32_t tableIndex)
{
	uint32_t key[2] = {((uint32_t*) startPoint)[0], ((uint32_t*) startPoint)[1]};

	for (uint32_t chainPos = 0; chainPos < chainLength - 1; chainPos++) // [sic]
	{
		rc4HashFunc(key, key);
		reduxKey(key, chainPos, tableIndex);
	}
	((uint32_t*) endpoint)[0] = key[0];
	((uint8_t*)  endpoint)[4] = ((uint8_t*) key)[4];
}

int main()
{
	uint32_t key[2] = {0, 0}, hash[2] = {0, 0};

	// Hash generation
	printf("Hash generation\n");
	excel97(key, hash);
	printf("excel97: %08x%02x\n", TO_BIG_ENDIAN_32(hash[0]), TO_LITTLE_ENDIAN_32(hash[1]));
	pdf40(key, hash);
	printf("pdf40:   %08x%02x\n", TO_BIG_ENDIAN_32(hash[0]), TO_LITTLE_ENDIAN_32(hash[1]));
	word97(key, hash);
	printf("word97:  %08x%02x\n", TO_BIG_ENDIAN_32(hash[0]), TO_LITTLE_ENDIAN_32(hash[1]));

	// Rainbow table generation
	printf("\nRainbow table generation\n");
	generateChain(excel97, key, hash, 1000, 0);
	printf("excel97(1000, 0): %08x%02x - %08x%02x\n", TO_BIG_ENDIAN_32(key[0]), TO_LITTLE_ENDIAN_32(key[1]), TO_BIG_ENDIAN_32(hash[0]), TO_LITTLE_ENDIAN_32(hash[1]));
	generateChain(pdf40, key, hash, 1000, 0);
	printf("pdf40(1000, 0):   %08x%02x - %08x%02x\n", TO_BIG_ENDIAN_32(key[0]), TO_LITTLE_ENDIAN_32(key[1]), TO_BIG_ENDIAN_32(hash[0]), TO_LITTLE_ENDIAN_32(hash[1]));
	generateChain(word97, key, hash, 1000, 0);
	printf("word97(1000, 0):  %08x%02x - %08x%02x\n", TO_BIG_ENDIAN_32(key[0]), TO_LITTLE_ENDIAN_32(key[1]), TO_BIG_ENDIAN_32(hash[0]), TO_LITTLE_ENDIAN_32(hash[1]));

	// Rainbow table generation benchmark
	TIMER_TYPE s, e;
	const uint32_t CHAINS = 100;

	printf("\nRainbow table generation benchmark\n");
	TIMER_FUNC(s);
	for (uint32_t i = 0; i < CHAINS; i++)
	{
		generateChain(excel97, key, hash, 1000, 0);
	}
	TIMER_FUNC(e);
	printf("excel97: %0.0f link/s\n", CHAINS * 999 / TIMER_DIFF(s, e));

	TIMER_FUNC(s);
	for (uint32_t i = 0; i < CHAINS; i++)
	{
		generateChain(pdf40, key, hash, 1000, 0);
	}
	TIMER_FUNC(e);
	printf("pdf40:   %0.0f link/s\n", CHAINS * 999 / TIMER_DIFF(s, e));

	TIMER_FUNC(s);
	for (uint32_t i = 0; i < CHAINS; i++)
	{
		generateChain(word97, key, hash, 1000, 0);
	}
	TIMER_FUNC(e);
	printf("word97:  %0.0f link/s\n", CHAINS * 999 / TIMER_DIFF(s, e));
	return 0;
}
